# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#          Roberto Alsina <roberto.alsina@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Windows main."""

from twisted.internet import defer

from ubuntuone_installer.gui.qt.main import windows
from ubuntuone_installer.tests import TestCase


class FakeLogger(object):

    """Fake Logger."""

    error_args = None
    debug_args = None
    info_args = None

    def error(self, *args, **kwargs):
        """Save the error."""
        self.error_args = (args, kwargs)

    def debug(self, *args, **kwargs):
        """Save the debug."""
        self.debug_args = (args, kwargs)

    def info(self, *args, **kwargs):
        """Save the info."""
        self.info_args = (args, kwargs)


class FakeGui(object):
    """Fake gui module."""

    def __init__(self):
        """Create a new instance."""
        self.main_window = None
        self.close = None
        self.installing = False
        self.called = []

    # pylint: disable=C0103
    def MainWindow(self, close_callback=lambda: None, installing=False):
        """Fake main window creation."""
        self.close = close_callback
        self.installing = installing
        self.called.append('MainWindow')
        return self.main_window
    # pylint: enable=C0103


class FakeWindow(object):
    """Fake a window."""

    def __init__(self):
        """Create a new instance."""
        self.shown = False

    def show(self):
        """Fake a shown window."""
        self.shown = True


class SuccessTestCase(TestCase):
    """Test the success callback."""

    @defer.inlineCallbacks
    def setUp(self):
        """Set up the diff tests."""
        yield super(SuccessTestCase, self).setUp()
        self.called = []
        self.start_name = 'start'
        self.update_name = 'update'
        self.stop_name = 'stop'
        self.gui = FakeGui()
        self.gui.main_window = FakeWindow()
        self.logger = FakeLogger()

        def fake_start(with_icon=False):
            """Fake the control panel start."""
            self.called.append((self.start_name, with_icon))
            return defer.succeed(None)

        def fake_check_updates(gui, logger):
            """Fake the check of updates."""
            self.called.append((self.update_name, gui, logger))
            return defer.succeed(None)

        def fake_stop():
            """Fake stop."""
            self.called.append((self.stop_name,))

        self.patch(windows, 'stop', fake_stop)
        self.start_cb = fake_start
        self.check_updates_cb = fake_check_updates

    @defer.inlineCallbacks
    def test_success_with_creds(self):
        """Test the success callback with creds."""
        yield windows.success_cb('creds', self.gui, False, self.logger,
                                 self.start_cb, self.check_updates_cb)
        self.assertEqual(self.start_name, self.called[0][0])
        self.assertFalse(self.called[0][1])
        self.assertEqual(self.update_name, self.called[1][0])
        self.assertEqual(self.gui, self.called[1][1])
        self.assertEqual(self.logger, self.called[1][2])
        self.assertEqual(self.stop_name, self.called[2][0])
        self.assertEqual(0, len(self.gui.called))

    @defer.inlineCallbacks
    def test_success_not_creds(self):
        """Test the success callback with no creds."""
        yield windows.success_cb(None, self.gui, True, self.logger,
                                 self.start_cb, self.check_updates_cb)
        self.assertTrue('MainWindow' in self.gui.called)
        self.assertTrue(self.gui.main_window.shown)


class ErrorTestCase(TestCase):
    """Test the success callback."""

    @defer.inlineCallbacks
    def setUp(self):
        """Set up the diff tests."""
        yield super(ErrorTestCase, self).setUp()
        self.called = []
        self.stop_name = 'stop'
        self.logger = FakeLogger()

        def fake_stop():
            """Fake stop."""
            self.called.append(self.stop_name)

        self.patch(windows, 'stop', fake_stop)

    def test_stop(self):
        """Test the error_cb"""
        windows.error_cb(Exception('test'), self.logger)
        self.assertTrue(self.stop_name in self.called)
